@@ -1,7 +1,7 @@
 package Data::Transmute;
 
-our $DATE = '2019-07-24'; # DATE
-our $VERSION = '0.030'; # VERSION
+our $DATE = '2019-08-22'; # DATE
+our $VERSION = '0.031'; # VERSION
 
 use 5.010001;
 use strict;
@@ -79,7 +79,22 @@ sub _rule_transmute_array_elems {
     my $data = $args{data};
     return unless ref $data eq 'ARRAY';
 
+    my $idx = -1;
+  ELEM:
     for my $el (@$data) {
+        $idx++;
+        if (defined $args{index_is}) {
+            next ELEM unless $idx == $args{index_is};
+        }
+        if (defined $args{index_in}) {
+            next ELEM unless grep { $idx == $_ } @{ $args{index_in} };
+        }
+        if (defined $args{index_match}) {
+            next ELEM unless $idx =~ $args{index_match};
+        }
+        if (defined $args{index_filter}) {
+            next ELEM unless $args{index_filter}->(index=>$idx, array=>$data, rules=>$args{rules});
+        }
         $el = transmute_data(
             data => $el,
             rules => $args{rules},
@@ -93,6 +108,10 @@ sub _rulereverse_transmute_array_elems {
 
     [transmute_array_elems => {
         rules => reverse_rules(rules => $args{rules}),
+        (index_is     => $args{index_is})     x !!(exists $args{index_is}),
+        (index_in     => $args{index_in})     x !!(exists $args{index_in}),
+        (index_match  => $args{index_match})  x !!(exists $args{index_match}),
+        (index_filter => $args{index_filter}) x !!(exists $args{index_filter}),
     }];
 }
 
@@ -102,9 +121,22 @@ sub _rule_transmute_hash_values {
     my $data = $args{data};
     return unless ref $data eq 'HASH';
 
-    for my $k (keys %$data) {
-        $data->{$k} = transmute_data(
-            data => $data->{$k},
+  KEY:
+    for my $key (keys %$data) {
+        if (defined $args{key_is}) {
+            next KEY unless $key eq $args{key_is};
+        }
+        if (defined $args{key_in}) {
+            next KEY unless grep { $key eq $_ } @{ $args{key_in} };
+        }
+        if (defined $args{key_match}) {
+            next KEY unless $key =~ $args{key_match};
+        }
+        if (defined $args{key_filter}) {
+            next KEY unless $args{key_filter}->(key=>$key, hash=>$data, rules=>$args{rules});
+        }
+        $data->{$key} = transmute_data(
+            data => $data->{$key},
             rules => $args{rules},
         );
     }
@@ -116,6 +148,10 @@ sub _rulereverse_transmute_hash_values {
 
     [transmute_hash_values => {
         rules => reverse_rules(rules => $args{rules}),
+        (key_is     => $args{key_is})     x !!(exists $args{key_is}),
+        (key_in     => $args{key_in})     x !!(exists $args{key_in}),
+        (key_match  => $args{key_match})  x !!(exists $args{key_match}),
+        (key_filter => $args{key_filter}) x !!(exists $args{key_filter}),
     }];
 }
 
@@ -174,7 +210,7 @@ Data::Transmute - Transmute (transform) data structure using rules data
 
 =head1 VERSION
 
-This document describes version 0.030 of Data::Transmute (from Perl distribution Data-Transmute), released on 2019-07-24.
+This document describes version 0.031 of Data::Transmute (from Perl distribution Data-Transmute), released on 2019-08-22.
 
 =head1 SYNOPSIS
 
@@ -187,9 +223,10 @@ This document describes version 0.030 of Data::Transmute (from Perl distribution
      data => \@data,
      rules => [
 
+         # CREATING HASH KEY
 
          # this rule only applies when data is a hash, when data is not a hash
-         # this will will do nothing. create a single new hash key, error if key
+         # this will do nothing. create a single new hash key, error if key
          # already exists.
          [create_hash_key => {name=>'foo', value=>1}],
 
@@ -202,9 +239,11 @@ This document describes version 0.030 of Data::Transmute (from Perl distribution
          [create_hash_key => {name=>'baz', value=>3, replace=>1}],
 
 
+         # RENAMING HASH KEY
+
          # this rule only applies when data is a hash, when data is not a hash
-         # this will will do nothing. rename a single key, error if old name
-         # doesn't exist or new name exists.
+         # this will do nothing. rename a single key, error if old name doesn't
+         # exist or new name exists.
          [rename_hash_key => {from=>'qux', to=>'quux'}],
 
          # rename another key, but this time ignore if old name doesn't exist
@@ -212,23 +251,48 @@ This document describes version 0.030 of Data::Transmute (from Perl distribution
          [rename_hash_key => {from=>'corge', to=>'grault', ignore_missing_from=>1, replace=>1}],
 
 
+         # DELETING HASH KEY
+
          # this rule only applies when data is a hash, when data is not a hash
-         # this will will do nothing. delete a single key, will noop if key
-         # already doesn't exist.
+         # this will do nothing. delete a single key, will noop if key already
+         # doesn't exist.
          [delete_hash_key => {name=>'garply'}],
 
 
-         # this rule only applies when data is an arrayref, when data is not a
-         # array this will will do nothing. for each array element, apply
-         # transmute rules to it.
+         # APPLYING (SUB)RULES TO ARRAY ELEMENTS
+
+         # this rule only applies when data is an arrayref, when data is not an
+         # array this will do nothing. for each array element, apply transmute
+         # rules to it.
          [transmute_array_elems => {rules => [...]}],
 
+         # you can select only certain elements to transmute by using one+ of:
+         # index_is, index_in, index_match, index_filter.
+         [transmute_array_elems => {
+              #index_is => 1,              # only transmute 2nd element (index is 0-based)
+              #index_in => [0,1,2],        # only transmute the first 3 elements
+              #index_match => qr/.../,     # only transmute elements where the index matches a regex
+              #index_filter => sub{...},   # only transmute elements where $filter->(index=>$index) returns true
+              rules => [...],
+          }],
+
+
+         # APPLYING (SUB)RULES TO HASH VALUES
 
          # this rule only applies when data is a hashref, when data is not a
-         # hash this will will do nothing. for each hash value, apply transmute
-         # rules to it.
+         # hash this will do nothing. for each hash value, apply transmute rules
+         # to it.
          [transmute_hash_values => {rules => [...]}],
 
+         # you can select only certain keys to transmute by using one+ of:
+         # key_is, key_in, key_match, key_filter.
+         [transmute_hash_values => {
+              #key_is => 'foo',          # only transmute value of key 'foo'
+              #key_in => ['foo', 'bar'], # only transmute value of keys 'foo', 'bar'
+              #key_match => qr/.../,     # only transmute value of keys that match a regex
+              #key_filter => sub{...},   # only transmute value of keys where $filter->(key=>$key) returns true
+              rules => [...],
+          }],
 
      ],
  );
@@ -249,6 +313,119 @@ Each rule specification: [$funcname, \%args]
 
 \%args: a special arg will be inserted: C<data>.
 
+=head2 create_hash_key
+
+This rule only applies when data is a hash, when data is not a hash this will do
+nothing. Create a single new hash key, error if key already exists.
+
+Known arguments (C<*> means required):
+
+=over
+
+=item * name*
+
+=item * value*
+
+=item * ignore
+
+Bool. If set to true, will ignore/noop if key already exists. This is like
+INSERT IGNORE (INSERT OR IGNORE) in SQL.
+
+=item * replace
+
+Bool. If set to true, will replace existing keys. This is like REPLACE INTO in
+SQL.
+
+=back
+
+=head2 rename_hash_key
+
+This rule only applies when data is a hash, when data is not a hash this will do
+nothing. Rename a single key, error if old name doesn't exist or new name
+exists.
+
+Known arguments (C<*> means required):
+
+=over
+
+=item * from*
+
+=item * to*
+
+=item * ignore_missing_from
+
+Bool. If set to true, will noop (instead of error) if old name doesn't exist.
+
+=item * replace
+
+Bool. If set to true, will overwrite (instead of error) when target key already
+exists.
+
+=back
+
+=head2 delete_hash_key
+
+This rule only applies when data is a hash, when data is not a hash this will do
+nothing. Delete a single key, will noop if key already doesn't exist.
+
+Known arguments (C<*> means required):
+
+=over
+
+=item * name*
+
+=back
+
+=head2 transmute_array_elems
+
+This rule only applies when data is an arrayref, when data is not an array this
+will do nothing. for each array element, apply transmute rules to it.
+
+Known arguments (C<*> means required):
+
+=over
+
+=item * rules*
+
+=item * index_is
+
+=item * index_in
+
+=item * index_match
+
+=item * index_filter
+
+Coderef. Only transmute elements where $coderef->(index=>$index) is true. Aside
+from C<index>, the coderef will also receive these arguments: C<rules> (the
+rule), C<array> (the array).
+
+=back
+
+=head2 transmute_hash_values
+
+This rule only applies when data is a hashref, when data is not a hash this will
+do nothing. For each hash value, apply transmute rules to it.
+
+Known arguments (C<*> means required):
+
+=over
+
+=item * rules*
+
+=item * key_is
+
+=item * key_in
+
+=item * key_match
+
+=item * key_filter
+
+Coderef. Only transmute value of keys where $coderef->(key=>$key) is true. Aside
+from C<key>, the coderef will also receive these arguments: C<rules> (the rule),
+C<hash> (the hash).
+
+=back
+
 =head1 FUNCTIONS
 
 =head2 transmute_data
@@ -262,6 +439,16 @@ argument, which will be modified in-place (so you'll need to clone it first if
 you don't want to modify the original data). Rules is specified in C<rules>
 argument.
 
+Known arguments (C<*> means required):
+
+=over
+
+=item * data*
+
+=item * rules*
+
+=back
+
 =head2 reverse_rules
 
 Usage:
@@ -270,16 +457,23 @@ Usage:
 
 Create a reverse rules, die on failure.
 
+Known arguments (C<*> means required):
+
+=over
+
+=item * rules*
+
+=back
+
 =head1 TODOS
 
 Check arguments (DZP:Rinci::Wrap?).
 
-Undo?
-
 Function to mass rename keys (by regex substitution, prefix, custom Perl code,
-...).
+...). But this cannot produce reverse of rule.
 
-Function to mass delete keys (by regex, prefix, ...).
+Function to mass delete keys (by regex, prefix, ...). But this cannot produce
+reverse of rule.
 
 =head1 HOMEPAGE
 
